import { DB } from 'kysely-codegen';
import { Kysely, SqliteDialect } from 'kysely';
import Database from 'better-sqlite3';
import jwt, { JwtPayload } from 'jsonwebtoken';

import { NextRequest } from 'next/server';
import { CandidateSchema } from '@/entities/Canditate';
import { UserSchema } from '@/entities/User';

let db: Kysely<DB> | null = null;

function initDb() {
  if (!db) {
    db = new Kysely<DB>({
      dialect: new SqliteDialect({
        database: async () => new Database(process.env.DATABASE_URL),
      }),
    });
  }
  return db;
}
function authenticate(req: NextRequest): {
  jwt: (JwtPayload & { userId: number }) | null;
  error: string | null;
} {
  const authHeader = req.headers.get('Authorization');
  if (!authHeader) {
    return { jwt: null, error: 'Authorization header is missing' };
  }
  const token = authHeader.split(' ')[1];
  if (!token) {
    return { jwt: null, error: 'Bearer token is missing' };
  }
  try {
    const decoded = jwt.verify(token, process.env.JWT_SECRET) as JwtPayload & {
      userId: number;
    };
    return { jwt: decoded, error: null };
  } catch (error) {
    return { jwt: null, error: `Invalid token, ${error}` };
  }
}

async function getUserById(id: number) {
  try {
    return UserSchema.parse(
      await initDb()
        .selectFrom('users')
        .selectAll()
        .where('id', '=', id)
        .executeTakeFirst(),
    );
  } catch (error) {
    throw new Response('Internal server error:  ' + error, {
      status: 500,
    });
  }
}
async function getCandidateById(id: number) {
  try {
    return CandidateSchema.parse(
      await initDb()
        .selectFrom('candidates')
        .selectAll()
        .where('id', '=', id)
        .executeTakeFirst(),
    );
  } catch (error) {
    throw new Response('Internal server error:  ' + error, {
      status: 500,
    });
  }
}
async function updateVotedStatus(id: number) {
  try {
    await initDb()
      .updateTable('users')
      .set({ voted: 1 })
      .where('id', '=', id)
      .execute();
  } catch (error) {
    throw new Response('Internal server error:  ' + error, {
      status: 500,
    });
  }
}
async function updateVotesCount(id: number) {
  try {
    await initDb()
      .updateTable('candidates')
      .set((eb) => ({
        votes: eb('votes', '+', 1),
      }))
      .where('id', '=', id)
      .execute();
  } catch (error) {
    throw new Response('Internal server error:  ' + error, {
      status: 500,
    });
  }
}

export async function GET() {
  try {
    if (!db) {
      db = new Kysely<DB>({
        dialect: new SqliteDialect({
          database: async () => new Database(process.env.DATABASE_URL),
        }),
      });
    }

    const candidates = await db.selectFrom('candidates').selectAll().execute();
    if (!candidates) {
      return new Response('Data not found', {
        status: 404,
      });
    }

    return new Response(JSON.stringify(candidates), {
      headers: { 'Content-Type': 'application/json' },
      status: 200,
    });
  } catch (error) {
    console.error('Error in GET request:', error);
    return new Response('Internal Server Error', {
      status: 500,
    });
  }
}

export async function POST(req: NextRequest) {
  try {
    const { jwt, error } = authenticate(req);
    if (error || !jwt) {
      return new Response(error, {
        status: 401,
      });
    }

    const dbUser = await getUserById(jwt.userId);
    if (dbUser.voted) {
      return new Response('User already voted', {
        status: 403,
      });
    }

    await updateVotedStatus(jwt.userId);
    const body = await req.json();
    await updateVotesCount(body.id);
    const dbCandidate = await getCandidateById(body.id);

    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    const { votes, ...candidateWithoutVotes } =
      CandidateSchema.parse(dbCandidate);
    return new Response(JSON.stringify(candidateWithoutVotes), {
      headers: { 'Content-Type': 'application/json' },
      status: 200,
    });
  } catch (error) {
    console.error('Error in POST request:', error);
    return new Response('Internal Server Error', {
      status: 500,
    });
  }
}
