import { DB } from 'kysely-codegen';
import { Kysely, SqliteDialect } from 'kysely';
import Database from 'better-sqlite3';
import jwt, { JwtPayload } from 'jsonwebtoken';
import { NextRequest } from 'next/server';
import { UserSchema } from '@/entities/User';
import { UserUpdate, UserUpdateSchema } from '@/entities/UserUpdate';

let db: Kysely<DB> | null = null;

function initDb() {
  if (!db) {
    db = new Kysely<DB>({
      dialect: new SqliteDialect({
        database: async () => new Database(process.env.DATABASE_URL),
      }),
    });
  }
  return db;
}
function authenticate(req: NextRequest): {
  jwt: (JwtPayload & { userId: number }) | null;
  error: string | null;
} {
  const authHeader = req.headers.get('Authorization');
  if (!authHeader) {
    return { jwt: null, error: 'Authorization header is missing' };
  }
  const token = authHeader.split(' ')[1];
  if (!token) {
    return { jwt: null, error: 'Bearer token is missing' };
  }
  try {
    const decoded = jwt.verify(token, process.env.JWT_SECRET) as JwtPayload & {
      userId: number;
    };
    return { jwt: decoded, error: null };
  } catch (error) {
    return { jwt: null, error: `Invalid token, ${error}` };
  }
}
async function getUserById(id: number) {
  try {
    return UserSchema.parse(
      await initDb()
        .selectFrom('users')
        .selectAll()
        .where('id', '=', id)
        .executeTakeFirst(),
    );
  } catch (error) {
    throw new Response('Internal server error:  ' + error, {
      status: 500,
    });
  }
}
async function updateUser(userId: number, updatedUserData: UserUpdate) {
  try {
    const updateObject: Partial<UserUpdate> = {
      name: updatedUserData.name,
      surname: updatedUserData.surname,
      email: updatedUserData.email,
    };
    if (updatedUserData.address) {
      updateObject.address = updatedUserData.address;
    }

    return await initDb()
      .updateTable('users')
      .set(updateObject)
      .where('id', '=', userId)
      .returningAll()
      .executeTakeFirst();
  } catch (error) {
    console.error('Error updating user:', error);
    throw new Response('Error updating user', {
      status: 500,
    });
  }
}
export async function GET(req: NextRequest) {
  try {
    const { jwt, error } = authenticate(req);
    if (error || !jwt) {
      return new Response(error, {
        status: 401,
      });
    }

    const user = await getUserById(jwt.userId);
    const userResponse = UserUpdateSchema.parse({
      name: user.name,
      surname: user.surname,
      email: user.email,
      address: user.address,
    });

    return new Response(JSON.stringify(userResponse), {
      headers: { 'Content-Type': 'application/json' },
      status: 200,
    });
  } catch (error) {
    console.error('Error in GET request:', error);
    return new Response('Internal Server Error', {
      status: 500,
    });
  }
}

export async function PUT(req: NextRequest) {
  try {
    const { jwt, error } = authenticate(req);
    if (error || !jwt) {
      return new Response(error, {
        status: 401,
      });
    }

    const body = await req.json();
    const result = UserUpdateSchema.safeParse(body);
    if (!result.success) {
      return new Response(result.error.message, {
        status: 400,
      });
    }
    const user: UserUpdate = result.data;

    return new Response(JSON.stringify(await updateUser(jwt.userId, user)), {
      headers: { 'Content-Type': 'application/json' },
      status: 200,
    });
  } catch (error) {
    console.error('Error in POST request:', error);
    return new Response('Internal Server Error', {
      status: 500,
    });
  }
}
