import { DB } from 'kysely-codegen';
import { Kysely, SqliteDialect } from 'kysely';
import Database from 'better-sqlite3';

import { NextRequest } from 'next/server';
import { LoginUserSchema } from '@/entities/LoginUserDto';
import { UserSchema } from '@/entities/User';
import { hashPassword, verifyPassword } from '@/utils/logIn/loginUtils';
import {
  encryptAESKeyWithRSA,
  encryptDataAES,
  generateRandomKey,
} from '@/utils/cipher/cryptoUtils';
import { NewUserResponseSchema } from '@/entities/NewUserResponseDto';
import jwt from 'jsonwebtoken';

let db: Kysely<DB> | null = null;

export async function POST(req: NextRequest) {
  try {
    if (!db) {
      db = new Kysely<DB>({
        dialect: new SqliteDialect({
          database: async () => new Database(process.env.DATABASE_URL),
        }),
      });
    }

    const body = await req.json();
    const user = LoginUserSchema.parse(body);
    if (!user) {
      return new Response('Bad request', {
        status: 400,
      });
    }
    const dbUser = UserSchema.parse(
      await db
        .selectFrom('users')
        .selectAll()
        .where('email', '=', user.email)
        .executeTakeFirst(),
    );

    const apiUser = UserSchema.parse(dbUser);
    if (apiUser === undefined) {
      return new Response('User not found', {
        status: 404,
      });
    }
    if (apiUser.loginAttempts <= 0) {
      const currentTime = Math.floor(Date.now() / 1000); // Convert milliseconds to seconds
      if (currentTime - apiUser.lastLoginAttempt >= 60) {
        // Allow the login attempt since enough time has passed
        // Reset login attempts to the default value and update the last login attempt time
        await db
          .updateTable('users')
          .set({ loginAttempts: 0, lastLoginAttempt: currentTime })
          .where('email', '=', apiUser.email)
          .executeTakeFirst();
      } else {
        // Enforce a delay and decrement login attempts
        const delay = 60 - (currentTime - apiUser.lastLoginAttempt);
        console.log(
          `Too many failed attempts. Please wait for ${delay} seconds.`,
        );
        return new Response(
          `Too many failed attempts. Please wait for ${delay} seconds.`,
          {
            status: 403,
          },
        );
      }
    }

    const symmetricKey = generateRandomKey();
    const userEncrypted = {
      name: encryptDataAES(apiUser.name, symmetricKey),
      surname: encryptDataAES(apiUser.surname, symmetricKey),
      email: encryptDataAES(apiUser.email, symmetricKey),
      password: encryptDataAES(hashPassword(apiUser.password), symmetricKey),
    };
    const encryptedSymmetricKey = encryptAESKeyWithRSA(
      symmetricKey,
      apiUser.publicKey,
    );

    const token = jwt.sign({ userId: dbUser.id }, process.env.JWT_SECRET, {
      expiresIn: '1h',
    });

    const response = {
      token: token,
      user: NewUserResponseSchema.parse(userEncrypted),
      symmetricKey: encryptedSymmetricKey,
    };

    if (verifyPassword(apiUser.password, user.password)) {
      await db
        .updateTable('users')
        .set({
          loginAttempts: 3,
          lastLoginAttempt: Math.floor(Date.now() / 1000),
        })
        .where('email', '=', apiUser.email)
        .executeTakeFirst();
      return new Response(JSON.stringify(response), {
        headers: { 'Content-Type': 'application/json' },
        status: 200,
      });
    } else {
      await db
        .updateTable('users')
        .set({
          loginAttempts: apiUser.loginAttempts - 1,
          lastLoginAttempt: Math.floor(Date.now() / 1000),
        })
        .where('email', '=', apiUser.email)
        .executeTakeFirst();
      return new Response(JSON.stringify({ msg: 'Bad credentials' }), {
        headers: { 'Content-Type': 'application/json' },
        status: 401,
      });
    }
  } catch (error) {
    console.error('Error in POST request:', error);
    return new Response('Internal Server Error', {
      status: 500,
    });
  }
}
