import { DB } from 'kysely-codegen';
import { Kysely, SqliteDialect } from 'kysely';
import Database from 'better-sqlite3';

import { PDFDocument, rgb, StandardFonts } from 'pdf-lib';
import { UserSchema } from '@/entities/User';
import {NextRequest} from "next/server";
import jwt, {JwtPayload} from "jsonwebtoken";

let db: Kysely<DB> | null = null;

function initDb() {
  if (!db) {
    db = new Kysely<DB>({
      dialect: new SqliteDialect({
        database: async () => new Database(process.env.DATABASE_URL),
      }),
    });
  }
  return db;
}

function authenticate(req: NextRequest) {
  const authHeader = req.headers.get('Authorization');
  if (!authHeader) {
    throw new Response('Authorization header is missing', {
      status: 401,
    });
  }
  const token = authHeader.split(' ')[1];
  if (!token) {
    throw new Response('Bearer token is missing', {
      status: 401,
    });
  }
  try {
    const decoded = jwt.verify(token, process.env.JWT_SECRET) as JwtPayload & {
      userId: number;
    };
    console.log(decoded);
    return decoded;
  } catch (error) {
    throw new Response('Invalid token', {
      status: 403,
    });
  }
}

async function getUserById(id: number) {
  try {
    return UserSchema.parse(
      await initDb()
        .selectFrom('users')
        .selectAll()
        .where('id', '=', id)
        .executeTakeFirst(),
    );
  } catch (error) {
    throw new Response('Internal server error:  ' + error, {
      status: 500,
    });
  }
}

export async function GET(req: NextRequest) {
  try {
    const jwt = authenticate(req);
    console.log(jwt.userId);
    // Find voter in the database
    const voter = await getUserById(jwt.userId);
    console.log('Voter found');
    console.log(voter.name);
    if (!voter) {
      return new Response('Data not found', {
        status: 404,
      });
    }

    // Generate a simple PDF
    const pdfContent = `Voting certificate\n\nPresidential elections - 2023\n\nThank you for your vote!\n\nVoter: ${voter.name} ${voter.surname}\n\n\nSUPER LEGIT CERTIFICATE`;
    const pdfBytes = await createPdf(pdfContent);

    const fileName = 'VotingCertificate.pdf';

    const headers = {
      'Content-Type': 'application/pdf',
      'Content-Disposition': `attachment; filename=${fileName}`,
    };

    return new Response(pdfBytes, {
      headers,
      status: 200,
    });
  } catch (error) {
    console.error('Error in GET request:', error);
    return new Response('Internal Server Error', {
      status: 500,
    });
  }
}
async function createPdf(content: string): Promise<Uint8Array> {
  const pdfDoc = await PDFDocument.create();
  const page = pdfDoc.addPage();
  const { width, height } = page.getSize();

  const font = await pdfDoc.embedFont(StandardFonts.Helvetica);
  page.drawText(content, {
    x: 50,
    y: height - 50,
    font,
    color: rgb(0, 0, 0),
  });

  const pdfBytes = await pdfDoc.save();

  return pdfBytes;
}
