import { DB } from 'kysely-codegen';
import { Kysely, SqliteDialect } from 'kysely';
import Database from 'better-sqlite3';
import jwt from 'jsonwebtoken';
import {
  generateRandomKey,
  encryptDataAES,
  encryptAESKeyWithRSA,
} from '@/utils/cipher/cryptoUtils';
import { NextRequest } from 'next/server';
import { checkPasswordInFile, hashPassword } from '@/utils/logIn/loginUtils';
import { NewUserRequestSchema } from '@/entities/NewUserRequestDto';
import { NewUserResponseSchema } from '@/entities/NewUserResponseDto';
import { UserSchema } from '@/entities/User';

let db: Kysely<DB> | null = null;

export async function POST(req: NextRequest) {
  try {
    if (!db) {
      db = new Kysely<DB>({
        dialect: new SqliteDialect({
          database: async () => new Database(process.env.DATABASE_URL),
        }),
      });
    }

    const body = await req.json();
    const publicKey = body.publicKey;
    const result = NewUserRequestSchema.safeParse(body.user);
    if (!result.success) {
      return new Response(result.error.message, {
        status: 400,
      });
    }
    const user = result.data;

    if (await checkPasswordInFile(user.password))
      return new Response(
        'Your password is a common password, you should change it',
        {
          status: 400,
        },
      );

    const userHashed = {
      name: user.name,
      surname: user.surname,
      email: user.email,
      password: hashPassword(user.password),
    };

    let createdUser;
    try {
      createdUser = UserSchema.parse(
        await db
          .insertInto('users')
          .values({
            ...userHashed,
            publicKey: publicKey,
          })
          .returningAll()
          .executeTakeFirst(),
      );
    } catch (error) {
      return new Response('Username with this email already exists', {
        status: 401,
      });
    }

    const symmetricKey = generateRandomKey();
    const userEncrypted = {
      name: encryptDataAES(user.name, symmetricKey),
      surname: encryptDataAES(user.surname, symmetricKey),
      email: encryptDataAES(user.email, symmetricKey),
      password: encryptDataAES(hashPassword(user.password), symmetricKey),
    };
    const encryptedSymmetricKey = encryptAESKeyWithRSA(symmetricKey, publicKey);

    const token = jwt.sign({ userId: createdUser.id }, process.env.JWT_SECRET, {
      expiresIn: '1h',
    });

    const response = {
      token: token,
      user: NewUserResponseSchema.parse(userEncrypted),
      symmetricKey: encryptedSymmetricKey,
    };

    return new Response(JSON.stringify(response), {
      headers: { 'Content-Type': 'application/json' },
      status: 200,
    });
  } catch (error) {
    console.error('Error in POST request:', error);
    return new Response('Internal Server Error', {
      status: 500,
    });
  }
}
