package com.example.upbz5.service;

import com.example.upbz5.exception.DataEncryptionException;
import com.example.upbz5.exception.NotFoundException;
import com.example.upbz5.exception.ReservationConflictException;
import com.example.upbz5.models.Accommodation;
import com.example.upbz5.models.Reservation;
import com.example.upbz5.models.User;
import com.example.upbz5.repository.ReservationRepository;
import com.example.upbz5.rest.dto.DatesPair;
import com.example.upbz5.rest.dto.ReservationRequest;
import com.example.upbz5.rest.dto.ReservationResponse;
import com.example.upbz5.rest.dto.ReservationSecretData;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.pdfbox.pdmodel.font.PDType1Font;
import org.springframework.stereotype.Service;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.PublicKey;
import java.time.format.DateTimeFormatter;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDPageContentStream;
import org.apache.pdfbox.pdmodel.common.PDRectangle;

import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.*;

@Service
@Slf4j
@AllArgsConstructor
public class ReservationServiceImpl implements ReservationService{

    private ReservationRepository reservationRepository;

    private UserServiceImpl userService;

    private AccommodationsServiceImpl accommodationsService;

    @Override
    public void createReservation(ReservationRequest request) throws ReservationConflictException, NotFoundException {
        if(!request.getCheckOutDate().isAfter(request.getCheckInDate())){
            throw new ReservationConflictException("Check-out date should be after check-in date");
        }
        if (request.getCheckInDate().isBefore(LocalDate.now())){
            throw new ReservationConflictException("Can not create reservation for the past");
        }
        Reservation newReservation = new Reservation();
        newReservation.setUser(userService.getById(request.getUserId()));
        Accommodation accommodation = accommodationsService.getAccommodationById(request.getAccommodationId());
        List<DatesPair> reservedDates = accommodationsService.getReservationDates(accommodation);
        for (DatesPair datesPair : reservedDates) {
            if ((request.getCheckInDate().isAfter(datesPair.getDateFrom()) && request.getCheckInDate().isBefore(datesPair.getDateTo())) ||
                    (request.getCheckOutDate().isAfter(datesPair.getDateFrom()) && request.getCheckOutDate().isBefore(datesPair.getDateTo())) ||
                    (request.getCheckInDate().isBefore(datesPair.getDateFrom()) && request.getCheckOutDate().isAfter(datesPair.getDateTo())) ||
                    (request.getCheckInDate().isEqual(datesPair.getDateFrom()) || (request.getCheckOutDate().isEqual(datesPair.getDateTo())))) {
                throw new ReservationConflictException("There is a existing reservation with conflicting reservation dates.");
            }
        }

        newReservation.setCheckInDate(request.getCheckInDate());
        newReservation.setCheckOutDate(request.getCheckOutDate());
        newReservation.setAccommodation(accommodation);
        newReservation.setGuestsCount(request.getGuestsCount());
        newReservation.setTotalPrice(request.getCheckInDate().until(request.getCheckOutDate(), ChronoUnit.DAYS) * accommodation.getPrice());
        Timestamp currentTimestamp = new Timestamp(System.currentTimeMillis());
        TimeZone bratislavaTimeZone = TimeZone.getTimeZone("Europe/Bratislava");
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(currentTimestamp.getTime());
        calendar.setTimeZone(bratislavaTimeZone);
        newReservation.setReservationDate(new Timestamp(calendar.getTimeInMillis()));
        reservationRepository.save(newReservation);
    }

    @Override
    public ReservationSecretData reservationDetail(Long id) throws NotFoundException{
        Optional<Reservation> optionalReservation = reservationRepository.findById(id);
        if (optionalReservation.isEmpty()){
            throw new NotFoundException("Reservation with this id:" + id + " doesn't exist");
        }
        Reservation reservation = optionalReservation.get();
        ReservationResponse response = ReservationResponse.builder()
                .id(reservation.getId())
                .userId(reservation.getUser().getId())
                .userFullName(reservation.getUser().getName() + " " + reservation.getUser().getSurname())
                .userEmail(reservation.getUser().getEmail())
                .accommodation(reservation.getAccommodation())
                .checkInDate(reservation.getCheckInDate())
                .checkOutDate(reservation.getCheckOutDate())
                .guestsCount(reservation.getGuestsCount())
                .totalPrice(reservation.getTotalPrice())
                .reservationDate(reservation.getReservationDate().toLocalDateTime().format(DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss")))
                .build();
        byte[] pdf;
        try {
            pdf = generatePDF(response);
        }
        catch (Exception e){
            throw new DataEncryptionException("Error occurred while creating pdf");
        }
        Map<String, String> pdfE;
        try {
            String key = reservation.getUser().getPublicKey();
            PublicKey publicKey = userService.getPublicKeyFromString(key);
            pdfE = userService.encrypt(pdf, publicKey);
        }
        catch (Exception e){
            throw new DataEncryptionException("Error occured while encrypting the pdf");
        }
        ReservationSecretData secretResponse = new ReservationSecretData();
        secretResponse.setPdf(pdfE.get("encryptedPdfData"));
        secretResponse.setEncryptedAes(pdfE.get("encryptedAesKey"));
        return secretResponse;
    }

    @Override
    public List<ReservationResponse> getUserReservations(Long userId) throws NotFoundException{
        User user = userService.getById(userId);
        List<Reservation> userReservations = reservationRepository.findByUser(user);
        List<ReservationResponse> responses = new ArrayList<>();
        for (Reservation reservation: userReservations) {
            responses.add(ReservationResponse.builder()
                    .id(reservation.getId())
                    .userId(reservation.getUser().getId())
                    .userFullName(reservation.getUser().getName() + " " + reservation.getUser().getSurname())
                    .userEmail(reservation.getUser().getEmail())
                    .accommodation(reservation.getAccommodation())
                    .checkInDate(reservation.getCheckInDate())
                    .checkOutDate(reservation.getCheckOutDate())
                    .guestsCount(reservation.getGuestsCount())
                    .totalPrice(reservation.getTotalPrice())
                    .reservationDate(reservation.getReservationDate().toLocalDateTime().format(DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss")))
                    .build());
        }
    return responses;
    }

    public static byte[] generatePDF(ReservationResponse reservation) throws IOException {
        try (PDDocument document = new PDDocument()) {
            PDPage page = new PDPage(PDRectangle.A4);
            document.addPage(page);

            try (PDPageContentStream contentStream = new PDPageContentStream(document, page)) {
                contentStream.setFont(PDType1Font.HELVETICA_BOLD, 18);

                contentStream.beginText();
                contentStream.newLineAtOffset(50, 700);

                contentStream.setLeading(14.5f);
                contentStream.showText("RESERVATION CONFIRMATION");
                contentStream.newLine();
                contentStream.newLine();

                contentStream.setFont(PDType1Font.HELVETICA_BOLD, 12);
                contentStream.showText("Reservation ID: " + reservation.getId().toString());
                contentStream.newLine();

                contentStream.showText("Full Name: " + reservation.getUserFullName());
                contentStream.newLine();

                contentStream.showText("Email: " + reservation.getUserEmail());
                contentStream.newLine();

                contentStream.showText("Accommodation name: " + reservation.getAccommodation().getName());
                contentStream.newLine();

                contentStream.showText("Accommodation address: " + reservation.getAccommodation().getAddress());
                contentStream.newLine();

                contentStream.showText("Check-In Date: " + reservation.getCheckInDate().format(DateTimeFormatter.ofPattern("dd.MM.yyyy")));
                contentStream.newLine();

                contentStream.showText("Check-Out Date: " + reservation.getCheckOutDate().format(DateTimeFormatter.ofPattern("dd.MM.yyy")));
                contentStream.newLine();

                contentStream.showText("Guests Count: " + reservation.getGuestsCount());
                contentStream.newLine();

                contentStream.showText("Total Price: " + reservation.getTotalPrice());
                contentStream.newLine();

                contentStream.showText("Reservation Creation Date: " + reservation.getReservationDate());
                contentStream.newLine();
                contentStream.newLine();

                contentStream.setFont(PDType1Font.HELVETICA_BOLD, 18);
                contentStream.showText("PAYMENT INFORMATION");
                contentStream.newLine();
                contentStream.newLine();

                contentStream.setFont(PDType1Font.HELVETICA_BOLD, 12);
                contentStream.showText("IBAN: SK45 0000 0055 1245");
                contentStream.newLine();

                contentStream.showText("Variable symbol: 1247845" + reservation.getId());
                contentStream.newLine();

                contentStream.showText("Total: " + reservation.getTotalPrice());
                contentStream.newLine();

                contentStream.showText("Payment deadline: " + reservation.getCheckInDate().minusDays(1).format(DateTimeFormatter.ofPattern("dd.MM.yyyy")));
                contentStream.endText();
            }
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            document.save(byteArrayOutputStream);
            return byteArrayOutputStream.toByteArray();
        }
    }

    @Override
    public Long getReservationOwner(Long reservationId){
        Optional<Reservation> optionalReservation = reservationRepository.findById(reservationId);
        if (optionalReservation.isEmpty()){
            throw new NotFoundException("Reservation with this id:" + reservationId + " doesn't exist");
        }
        Reservation reservation = optionalReservation.get();
        return reservation.getUser().getId();
    }
}
