package com.example.upbz5.service;

import com.example.upbz5.exception.LoginException;
import com.example.upbz5.exception.NotFoundException;
import com.example.upbz5.exception.RegistrationException;
import com.example.upbz5.models.User;
import com.example.upbz5.repository.UserRepository;
import com.example.upbz5.rest.dto.*;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.impl.TextCodec;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
@Slf4j
@AllArgsConstructor
public class UserServiceImpl implements UserService {

    private final UserRepository repository;

    private final BCryptPasswordEncoder bCryptPasswordEncoder;

    @Override
    public UserDto registerUser(UserRegistrationRequest registrationRequest) throws RegistrationException{
        if (registrationRequest.getEmail() == null || registrationRequest.getPublicKey() == null
                || registrationRequest.getPassword() == null || registrationRequest.getEmail().isEmpty()){
            throw  new RegistrationException("Email and public key has to be provided");
        }
        if(!emailCheck(registrationRequest.getEmail())){
            throw new RegistrationException("Invalid email format");
        }
        Optional<User> existingUser = this.repository.findByEmail(registrationRequest.getEmail());
        if(existingUser.isPresent()) {
            throw new RegistrationException("User with this email already exist");
        }
        else{
            if(registrationRequest.getPassword().length() < 8){
                throw new RegistrationException("Password should be at least 8 characters long");
            }
            else if (!safePasswordCheck(registrationRequest.getPassword())){
                throw new RegistrationException("Password should contain at least: 1 uppercase, 1 lowercase, 1 digit and one of special characters: @#$%^&+=");
            }
            User user = new User();
            user.setName(registrationRequest.getName());
            user.setSurname(registrationRequest.getSurname());
            user.setEmail(registrationRequest.getEmail());
            user.setPublicKey(registrationRequest.getPublicKey());
            user.setPassword(bCryptPasswordEncoder.encode(registrationRequest.getPassword()));
            this.repository.save(user);
            return new UserDto(user.getId(),user.getName(), user.getSurname(), user.getEmail(), null);
        }
    }

    @Override
    public UserDto loginUser(UserLoginRequest loginRequest) throws NotFoundException {
        Optional<User> existingUser = this.repository.findByEmail(loginRequest.getEmail());
        if(existingUser.isEmpty()) {
            throw new NotFoundException("Email or password is not correct");
        }
        else if(!bCryptPasswordEncoder.matches(loginRequest.getPassword(), existingUser.get().getPassword())){
            throw new NotFoundException("Email or password is not correct");
        }
        else {
            String jwt = Jwts.builder()
                    .setSubject(existingUser.get().getId().toString())
                    .setExpiration(new Date(System.currentTimeMillis() + 30 * 60 * 1000))
                    .signWith(
                            SignatureAlgorithm.HS256,
                            TextCodec.BASE64.decode("Yn2kjibddFAWtnPJ2AFlL8WXmohJMCvigQggaEypa5E=")
                    )
                    .compact();
            return new UserDto(existingUser.get().getId(), existingUser.get().getName(), existingUser.get().getSurname(), existingUser.get().getEmail(), jwt);
        }
        }

    @Override
    public ValidationDto isValidToken(String jwt) {
        try {
            Jwts.parser()
                    .setSigningKey("Yn2kjibddFAWtnPJ2AFlL8WXmohJMCvigQggaEypa5E=")
                    .parseClaimsJws(jwt)
                    .getBody();
            return new ValidationDto(true);
        }
        catch (Exception e){
            return new ValidationDto(false);
        }
    }

    public User getById(Long id) throws NotFoundException{
        Optional<User> user = repository.findById(id);
        if(user.isEmpty()){
            throw new NotFoundException("User with this id:" + id + " doesn't exist");
        }
        return user.get();
    }

    @Override
    public Long getUserIdByToken(String jwt){
        try {
            Claims claims = Jwts.parser()
                    .setSigningKey("Yn2kjibddFAWtnPJ2AFlL8WXmohJMCvigQggaEypa5E=")
                    .parseClaimsJws(jwt)
                    .getBody();
            return Long.parseLong(claims.getSubject());
        }
        catch (Exception e){
            throw new LoginException("Invalid token");
        }

    }

    public static boolean safePasswordCheck(String password) {
        String regex = "^(?=.*[0-9])" + "(?=.*[a-z])(?=.*[A-Z])" + "(?=.*[@#$%^&+=])" + "(?=\\S+$).{8,255}$";
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(password);
        return m.matches();
    }

    public static boolean emailCheck(String email) {
        String regex = "^(?=.{1,64}@)[A-Za-z0-9_-]+(\\.[A-Za-z0-9_-]+)*@" + "[^-][A-Za-z0-9-]+(\\.[A-Za-z0-9-]+)*(\\.[A-Za-z]{2,})$";
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(email);
        return m.matches();
    }

    @Override
    public PublicKey getPublicKeyFromString(String key) throws Exception {
        byte[] byteKey = Base64.getDecoder().decode(key);
        X509EncodedKeySpec X509publicKey = new X509EncodedKeySpec(byteKey);
        KeyFactory keyFactory = KeyFactory.getInstance("RSA");
        return keyFactory.generatePublic(X509publicKey);
    }

    @Override
    public Map<String, String> encrypt(byte[] pdfData, PublicKey publicKey) throws Exception {

        String generatedKey = generateRandomString();

        byte[] keyBytes = generatedKey.getBytes(StandardCharsets.UTF_8);

        SecretKey secretKey = new SecretKeySpec(keyBytes, "AES");

        Cipher symmetricCipher = Cipher.getInstance("AES");
        symmetricCipher.init(Cipher.ENCRYPT_MODE, secretKey);

        String pdfString = Arrays.toString(pdfData);

        byte[] encryptedBytes =symmetricCipher.doFinal(pdfString.getBytes(StandardCharsets.UTF_8));
        String before = Base64.getEncoder().encodeToString(encryptedBytes);

        Cipher cipher = Cipher.getInstance("RSA");
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        byte[] encryptedBytesRSA = cipher.doFinal(generatedKey.getBytes());

        Map<String, String> encryptedDataMap = new HashMap<>();
        encryptedDataMap.put("encryptedPdfData", before);
        encryptedDataMap.put("encryptedAesKey", Base64.getEncoder().encodeToString(encryptedBytesRSA));

        return encryptedDataMap;
    }

    public String generateRandomString() {
        return RandomStringUtils.randomAlphabetic(16);
    }
}
