import {Button, Grid, IconButton, InputAdornment, TextField, Tooltip, Typography} from "@mui/material";
import React, {useEffect, useState} from "react";
import {useFormik} from "formik";
import sha1 from "sha1";
import {toast} from "react-toastify";
import SearchIcon from '@mui/icons-material/Search';
import * as Yup from "yup";
import {AccommodationCard} from "./accommodation-card";
import {getCookie} from "../App";
import {useNavigate} from "react-router-dom";


const url = process.env.REACT_APP_API_URL
const validationSchema = Yup.object({
    name: Yup.string()
        .max(30, 'Name must be at most 30 characters')
        .matches(/^[a-zA-Z0-9áäčďéíĺľňóôŕšťúýžÁÄČĎÉÍĹĽŇÓÔŔŠŤÚÝŽ]+$/, 'Search can only contain letters and numbers'),

    minCapacity: Yup.number()
        .test('custom-test', 'Capacity must be greater then 0', function (value) {
            return value > 0 || value === undefined // For example, max capacity should be less than or equal to 100
        }),

});


export const AccommodationPage = () => {
    const navigate = useNavigate()
    const [disabledButton, setDisabledButton] = useState(true)
    const [accommodations, setAccommodation] = useState([])
    const [maxRequests, setMaxRequest] = useState(3);
    const [requestInterval, setRequestInterval] = useState(5000)
    const [requestCount, setRequestCount] = useState(0)
    const [lastRequestTime, setLastRequestTime] = useState(0)
    const formik = useFormik({
        initialValues: {
            name: '',
            minCapacity: undefined,
            address: undefined,
            maxPrice: undefined,
            minPrice: undefined
        },
        validationSchema: validationSchema,
        onSubmit: async (values) => {

            try {
                await fetchWithRateLimit(values)
            } catch (error) {

                if (error instanceof Error) {
                    toast.error(error.message); // Display the error message in a toast
                } else {
                    const errorMessage = await error.text();
                    const errorObject = JSON.parse(errorMessage);
                    toast.error(errorObject.error); // Display the custom backend exception message in a toast
                }
            }


        }
    });

    useEffect(() => {

        const fetchData = async () => {
            const token = getCookie('auth');
            try {
                const response = await fetch(url + '/accommodation/top', {
                        method: 'GET',

                        headers: {
                            'Authorization': `${token}`,
                        },
                    },
                )

                if (!response.ok) {
                    if (response.status === 401) {
                        navigate("/signin")
                    } else {
                        console.error(`HTTP error! Status: ${response.status}`);
                    }
                } else {
                    const result = await response.json();
                    setAccommodation(result.accommodations)
                }

            } catch (error) {
                console.error('Error fetching data:', error);
            }
        };

        fetchData();

    }, []);


    const fetchWithRateLimit = async (body) => {
        return new Promise((resolve, reject) => {
            const makeRequest = async () => {
                const currentTime = new Date().getTime();
                if (currentTime - lastRequestTime < requestInterval) {
                    setRequestCount(requestCount + 1);
                    setLastRequestTime(currentTime)
                    if (requestCount > maxRequests) {
                        reject(new Error('Too many requests, please try again later.'));
                        return;
                    }
                } else {
                    setRequestCount(1)
                    setLastRequestTime(currentTime)
                }
                try {
                    const token = getCookie('auth');
                    const response = await fetch(url + '/accommodation/filter', {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'Authorization': `${token}`
                        },
                        body: JSON.stringify(body)
                    });
                    const jsonData = await response.json();

                    if (!response.ok) {
                        if (response.status === 401) {
                            navigate("/signin")
                        } else {
                            toast.error(jsonData.message)
                        }
                    } else {
                        setAccommodation(jsonData.accommodations)
                    }
                } catch (error) {
                    reject(error);
                }
            };

            makeRequest();
        });
    };

    return (
        <Grid
            container
            spacing={2}
            direction="column"
            alignItems="center"
            style={{minHeight: '100vh', width: '100%'}}
        >
            <Grid item xs={12} sm={12} md={12}>
                <form onSubmit={formik.handleSubmit}>
                    <Grid container gridTemplateColumns="repeat(12, 1fr)" justifyContent="center"
                          alignItems="center" width='96vw' style={{marginTop: '5rem'}} spacing={2}>
                        <Grid item xs={12} md={5}>
                            <TextField
                                id="name"
                                name="name"
                                fullWidth={true}
                                value={formik.values.name}
                                onChange={(e) => {
                                    formik.setFieldValue("name", e.target.value)
                                    if (e.target.value !== "" && e.target.value !== undefined) {
                                        setDisabledButton(false)
                                    } else (
                                        setDisabledButton(true)
                                    )
                                }}
                                InputProps={{
                                    startAdornment: <SearchIcon/>,
                                }}
                                placeholder={"  Search accommodation"}
                                error={formik.touched.name && Boolean(formik.errors.name)}
                                helperText={formik.touched.name && formik.errors.name}
                            />

                        </Grid>
                        <Grid item xs={12} md={2}>
                            <TextField
                                id="minCapacity"
                                name="minCapacity"
                                fullWidth={true}
                                InputProps={{
                                    inputProps: {
                                        min: 1,
                                        max: 30
                                    },
                                }}
                                type="number"
                                value={formik.values.minCapacity}
                                onChange={formik.handleChange}
                                placeholder={"Capacity"}
                                error={formik.touched.minCapacity && Boolean(formik.errors.minCapacity)}
                                helperText={formik.touched.minCapacity && formik.errors.minCapacity}
                            />

                        </Grid>

                        <Grid item xs={12} md={1}>
                            <Button type='submit' fullWidth={true} disabled={disabledButton}
                                    variant="contained">
                                Search
                            </Button>
                        </Grid>
                    </Grid>
                </form>

                <Grid container gridTemplateColumns="repeat(12, 1fr)" justifyContent="center"
                      alignItems="center" width='96vw' style={{marginTop: '1rem', paddingBottom: "2rem"}} spacing={2}>
                    {accommodations.length > 0 && accommodations.map((accommodation) => {
                        return <AccommodationCard key={accommodation.id}
                                                  accommodation={accommodation}></AccommodationCard>
                    })}

                </Grid>


            </Grid>

        </Grid>
    )
}