import {useFormik} from "formik";
import React, {useState} from 'react';
import {Button, Grid, IconButton, InputAdornment, TextField, Tooltip, Typography} from "@mui/material";
import * as Yup from 'yup';
import sha1 from "sha1";
import {toast} from "react-toastify";
import InfoIcon from '@mui/icons-material/Info';
import {useNavigate} from "react-router-dom";

const validationSchema = Yup.object({
    email: Yup.string().email('Invalid email').required('Email is required').max(100, 'E-mail can have max 100 characters.'),
    name: Yup.string().required('Name is required').max(50, 'Name can have max 50 characters.'),
    surname: Yup.string().required('Surname is required').max(100, 'Surname can have max 100 characters.'),
    password: Yup.string().required('Password is required').matches(
        /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@#$%^&+=]).{8,}$/,
        'Must contain at least 8 characters, one uppercase, one lowercase, one number, and one special character @#$%^&+= '
    ).max(100, 'Password can have max 100 characters.'),
    publicKey: Yup.string().required('Public Key is required')
});

const url = process.env.REACT_APP_API_URL


export const RegistrationForm = () => {
    const [checked, setChecked] = useState(false)

    const navigate = useNavigate();
    const formik = useFormik({
        initialValues: {
            email: '',
            name: '',
            surname: '',
            password: '',
            publicKey: ''
        },
        validationSchema: validationSchema,
        onSubmit: async (values) => {
            const newPassword = values.password;
            const hashedPassword = sha1(newPassword).toUpperCase();
            try {
                const response = await fetch(`https://api.pwnedpasswords.com/range/${hashedPassword.substring(0, 5)}`);
                const data = await response.text();
                const compromisedPasswords = data.split('\n').map((line) => line.split(':')[0]);
                const invalid = compromisedPasswords.includes(hashedPassword.substring(5))

                if (invalid !== true) {
                    setChecked(true);
                    await postToBE(values)
                } else {
                    toast.error('This password is in public lists of passwords leaked from other sites.');
                    setChecked(false)
                }
            } catch (error) {
                toast.error('Error checking password');
            }


        },
    });

    const postToBE = async (values) => {
        try {
            const response = await fetch(url + '/api/register', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(values)
            });
            const jsonData = await response.json();
            if (!response.ok) {
                toast.error(jsonData.message)
            } else {
                navigate('/signin')
                formik.resetForm()
                toast.success('Registration Successful! Thank you for registering. You can now log in to your account.');
            }
        } catch (error) {
            if (error.response) {
                const errorData = await error.response.json();
                toast.error(errorData.message);
            } else if (error.request) {
                toast.error('No response received from server. ');
            } else {
                const errorData = await error.response.json();
                toast.error(errorData.message)
            }

        }
    }


    const submit = () => {

    }
    return (
        <Grid
            container
            spacing={2}
            direction="column"
            alignItems="center"
            justifyContent="center"
            style={{minHeight: '100vh', width: '100%'}}
        >
            <Grid item xs={12} sm={12} md={12}>
                <form onSubmit={formik.handleSubmit}>
                    <Grid container gridTemplateColumns="repeat(7, 1fr)" justifyContent="center"
                          alignItems="center" width='100%' spacing={2}>
                        <Grid item xs={7} md={7}>
                            <Typography variant="h5" fullWidth={true}>Registration</Typography>
                        </Grid>
                        <Grid item xs={7} md={7}>
                            <TextField
                                id="name"
                                name="name"
                                fullWidth={true}
                                label="Name"
                                value={formik.values.name}
                                onChange={formik.handleChange}
                                error={formik.touched.name && Boolean(formik.errors.name)}
                                helperText={formik.touched.name && formik.errors.name}
                            />
                        </Grid>
                        <Grid item xs={7} md={7}>
                            <TextField
                                id="surname"
                                name="surname"
                                label="Surname"
                                fullWidth={true}
                                value={formik.values.surname}
                                onChange={formik.handleChange}
                                error={formik.touched.surname && Boolean(formik.errors.surname)}
                                helperText={formik.touched.surname && formik.errors.surname}
                            />
                        </Grid>
                        <Grid item xs={7} md={7}>
                            <TextField
                                id="email"
                                name="email"
                                label="Email"
                                fullWidth={true}
                                value={formik.values.email}
                                onChange={formik.handleChange}
                                error={formik.touched.email && Boolean(formik.errors.email)}
                                helperText={formik.touched.email && formik.errors.email}
                            />
                        </Grid>

                        <Grid item xs={7} md={7}>
                            <TextField
                                id="password"
                                name="password"
                                label="Password"
                                type="password"
                                fullWidth={true}
                                value={formik.values.password}
                                onChange={formik.handleChange}
                                error={formik.touched.password && Boolean(formik.errors.password)}
                                helperText={formik.touched.password && formik.errors.password}
                            />
                        </Grid>
                        <Grid item xs={7} md={7}>
                            <TextField
                                id="publicKey"
                                name="publicKey"
                                label="Public Key"
                                type="publicKey"
                                fullWidth={true}
                                value={formik.values.publicKey}
                                onChange={formik.handleChange}
                                error={formik.touched.publicKey && Boolean(formik.errors.publicKey)}
                                helperText={formik.touched.publicKey && formik.errors.publicKey}
                                InputProps={{
                                    endAdornment: (
                                        <InputAdornment position="end">
                                            <Tooltip
                                                title="Please generate the key on  ' https://www.devglan.com/online-tools/rsa-encryption-decryption ' site and save the Private Key">
                                                <IconButton aria-label="Information">
                                                    <InfoIcon/>
                                                </IconButton>
                                            </Tooltip>
                                        </InputAdornment>
                                    ),
                                }}
                            />
                        </Grid>
                        <Grid item xs={7} md={7}>
                            <Button type="submit" fullWidth={true} variant="contained">
                                Submit
                            </Button>
                        </Grid>
                    </Grid>
                </form>
            </Grid>

        </Grid>
    );
};
