<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\WeakPassword;
use App\Utils\Security;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cookie;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Str;
use Tymon\JWTAuth\Facades\JWTAuth;
use Illuminate\Support\Facades\Validator;

class UsersController extends Controller
{
    use Security;

    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), $this->makeRules());

        if ($validator->fails()) {
            return response()->json([
                "message" => "Password must be at least 8 characters long and contain at least one uppercase letter, one lowercase letter, one number, and one special character",
            ], 400);
        }

        $user = User::create($request->all());

        $credentials = request(['email', 'password']);

        Auth::attempt($credentials);
        $user = Auth::user();
        $token = JWTAuth::fromUser($user);

        return response()->json([
            "message" => "User created successfully",
            "token" => $token,
            "user" => $user,
        ], 201);
    }

    public function login(Request $request)
    {
        if (Cookie::get('remember_token') != null) {
            $user = User::where('remember_token', Cookie::get('remember_token'))->first();

            if ($user != null) {
                $token = JWTAuth::fromUser($user);

                if (!$user->public_key == null) {
                    $user->public_key_fingerprint = $this->validatePublicKey($user->public_key)->getFingerprint('sha256');
                }

                return response()->json([
                    "message" => "Login succesfuly",
                    "token" => $token,
                    "user" => $user,
                ]);
            }
        }

        if (RateLimiter::tooManyAttempts(request()->ip(), 3)) {
            return response()->json([
                "message" => "Too many attempts, please try again later",
            ], 429);
        }

        $credentials = request(['email', 'password']);

        if (!auth()->attempt($credentials)) {
            RateLimiter::hit(request()->ip());
            return response()->json(['errors' => 'Invalid username and Password'], 401);
        }

        Auth::attempt($credentials);
        $user = Auth::user();
        $token = JWTAuth::fromUser($user);

        if ($request->remember_me == true) {
            if ($user->remember_token == null) {
                $user->setRememberToken(Str::random(60));
                $user->remember_token_expires_at = Carbon::now()->addDays(7);
                $user->save();
            }
        }

        if (!$user->public_key == null) {
            $user->public_key_fingerprint = $this->validatePublicKey($user->public_key)->getFingerprint('sha256');
        }

        return response()->json([
            "message" => "Login succesfuly",
            "token" => $token,
            "user" => $user,
        ])->withCookie(cookie('remember_token', $user->remember_token, 60 * 24 * 7));
    }

    public function refresh()
    {
        $token = JWTAuth::getToken();
        $newToken = JWTAuth::refresh($token);

        return response()->json([
            "token" => $newToken,
        ]);
    }

    public function logOut()
    {
        $token = JWTAuth::getToken();
        $token = JWTAuth::invalidate($token);
        $cookie = Cookie::forget('remember_token');

        return response()->json([
            "message" => "Logout succesfuly",
        ])->withCookie($cookie);
    }

    public function logOutFromAll()
    {
        $token = JWTAuth::getToken();
        $user = JWTAuth::toUser($token);
        $token = JWTAuth::invalidate($token);
        $cookie = Cookie::forget('remember_token');
        $user->setRememberToken(null);
        $user->save();

        return response()->json([
            "message" => "Logout succesfuly",
        ])->withCookie($cookie);
    }

    public function checkStrength(Request $request)
    {
        if (WeakPassword::where('password', $request->password)->first()) {
            return response()->json([
                "message" => "Password is in database of weak passwords",
            ], 400);
        }

        return response()->json([
            "message" => "Password is not in database of weak passwords",
        ], 200);
    }

    public function savePublicKey(Request $request)
    {
        $publicKey = $request->getContent();
        $publicKey = $this->validatePublicKey($publicKey);

        if ($publicKey == null) {
            return response()->json([
                "message" => "Invalid public key",
            ], 400);
        }

        $user = Auth::user();
        $user->public_key = $publicKey->toString('PKCS8');
        $user->save();

        $user->public_key_fingerprint = $publicKey->getFingerprint('sha256');

        return response()->json([
            'data' => $user
        ]);
    }

    private function makeRules()
    {
        return [
            'password' => [
                'required',
                'min:8',
                'regex:/[a-z]/',
                'regex:/[A-Z]/',
                'regex:/[0-9]/',
                'regex:/[!@#\$%\^&*()_-]/',
            ],
        ];
    }
}
