// stolen from https://gist.github.com/deiu/2c3208c89fbc91d23226

export function convertBinaryToPem(
  binaryData: ArrayBuffer,
  isPrivate: boolean = true,
) {
  const base64Cert = arrayBufferToBase64String(binaryData)
  let pemCert = `-----BEGIN RSA ${isPrivate ? 'PRIVATE' : 'PUBLIC'} KEY-----\n`
  let nextIndex = 0

  while (nextIndex < base64Cert.length) {
    if (nextIndex + 64 <= base64Cert.length) {
      pemCert += base64Cert.substring(nextIndex, nextIndex + 64) + '\n'
    } else {
      pemCert += base64Cert.substring(nextIndex) + '\n'
    }
    nextIndex += 64
  }
  pemCert += `-----END RSA ${isPrivate ? 'PRIVATE' : 'PUBLIC'} KEY-----\n`

  return pemCert
}

function arrayBufferToBase64String(arrayBuffer: ArrayBuffer) {
  const byteArray = new Uint8Array(arrayBuffer)
  let byteString = ''

  for (var i = 0; i < byteArray.byteLength; i++) {
    byteString += String.fromCharCode(byteArray[i])
  }

  return btoa(byteString)
}

/*
Convert a string into an ArrayBuffer
from https://developers.google.com/web/updates/2012/06/How-to-convert-ArrayBuffer-to-and-from-String
*/
function str2ab(str: string) {
  const buf = new ArrayBuffer(str.length)
  const bufView = new Uint8Array(buf)
  for (let i = 0, strLen = str.length; i < strLen; i++) {
    bufView[i] = str.charCodeAt(i)
  }
  return buf
}

const pemContentsRegex =
  /^-----BEGIN (RSA )?PRIVATE KEY-----([\s\S]+)-----END (RSA )?PRIVATE KEY-----\s*$/

/*
Import a PEM encoded RSA private key, to use for RSA-PSS signing.
Takes a string containing the PEM encoded key, and returns a Promise
that will resolve to a CryptoKey representing the private key.
*/
export function importPrivateKey(pem: string) {
  // fetch the part of the PEM string between header and footer
  const pemContents = pem.match(pemContentsRegex)?.[2]

  if (!pemContents) {
    throw new Error('Invalid PEM contents')
  }

  // base64 decode the string to get the binary data
  const binaryDerString = window.atob(pemContents.replace(/\s/g, ''))
  // convert from a binary string to an ArrayBuffer
  const binaryDer = str2ab(binaryDerString)

  return window.crypto.subtle.importKey(
    'pkcs8',
    binaryDer,
    {
      name: 'RSA-OAEP',
      hash: 'SHA-256',
    },
    false,
    ['decrypt'],
  )
}

export async function decrypt(data: ArrayBuffer, privateKey: CryptoKey) {
  const [len] = new Uint16Array(data.slice(0, 2))
  const encryptedKeyData = data.slice(2, 2 + len)
  const keyData = await window.crypto.subtle.decrypt(
    {
      name: 'RSA-OAEP',
    },
    privateKey,
    encryptedKeyData,
  )

  const [keyLen, ivLen, tagLen] = new Uint8Array(keyData.slice(0, 3))
  const key = keyData.slice(3, 3 + keyLen)
  const iv = keyData.slice(3 + keyLen, 3 + keyLen + ivLen)
  const tag = keyData.slice(3 + keyLen + ivLen, 3 + keyLen + ivLen + tagLen)
  const aesKey = await window.crypto.subtle.importKey(
    'raw',
    key,
    { name: 'AES-GCM' },
    false,
    ['decrypt'],
  )

  const encryptedData = data.slice(2 + len)

  // concat tag to encrypted data because that's what SubtleCrypto expects
  // https://stackoverflow.com/questions/75877831/php-openssl-encrypt-and-javascript-crypto-subtle-encrypt-have-different-output
  const encryptedDataWithTag = new Uint8Array(
    encryptedData.byteLength + tag.byteLength,
  )
  encryptedDataWithTag.set(new Uint8Array(encryptedData), 0)
  encryptedDataWithTag.set(new Uint8Array(tag), encryptedData.byteLength)

  return await window.crypto.subtle.decrypt(
    {
      name: 'AES-GCM',
      iv,
    },
    aesKey,
    encryptedDataWithTag,
  )
}

export const priceFormatter = new Intl.NumberFormat('sk-SK', {
  style: 'currency',
  currency: 'EUR',
})
